//
// deref.hh: pretty printing pointers
// Copyright (C) 2003, 2004 Akim Demaille <akim@epita.fr> and 
//                          Benoit Perrot <benoit@lrde.epita.fr>
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// It is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//
#ifndef MISC_DEREF_HH
# define MISC_DEREF_HH

# include <iostream>

# include "misc/contract.hh"

namespace misc
{

  class Deref
  {
  protected:
    Deref():
      postr_(0)
    {}
  public:
    static Deref&	instance()
    {
      static Deref	unique;
      return unique;
    }
    virtual ~Deref()
    {
    }

  public:
    template <class T>
    std::ostream&	operator<<(const T* t) const
    {
      assertion(postr_);
      (*postr_) << *t;
      return (*postr_);
    }
    template <class T>
    std::ostream&	operator<<(T* t) const
    {
      assertion(postr_);
      (*postr_) << *t;
      return (*postr_);
    }
    template <class T>
    std::ostream&	operator<<(const T& t) const
    {
      assertion(postr_);
      (*postr_) << t;
      return (*postr_);
    }
    template <class T>
    std::ostream&	operator<<(T& t) const
    {
      assertion(postr_);
      (*postr_) << t;
      return (*postr_);
    }

  public:
    void		ostr_set(std::ostream& ostr)
    {
      postr_ = &ostr;
    }

  protected:
    std::ostream*	postr_;
  };

  inline Deref&
  operator<< (std::ostream& ostr, Deref& d)
  {
    d.ostr_set(ostr);
    return d;
  }

} // namespace misc

# define deref	misc::Deref::instance()

#endif // !MISC_DEREF_HH
