//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2003, 2004, 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//

/** \file shell/cmd.hh
    \brief Declare Command class. */

#ifndef SHELL_CMD_HH
# define SHELL_CMD_HH

# include <list>
# include <string>

namespace shell
{
  /// Cmd Class.
  class Cmd
  {
  public:
    /// Command identifier.
    enum identifier_type
      {
	id_unknown,
	id_ambiguous,
	id_null,
	id_load,
	id_run,
	id_quit,
	id_continue,
	id_break,
	id_next,
	id_print,
	id_display,
	id_undisplay,
	id_dump,
	id_mem,
	id_backtrace,
	id_nop_after_branch,
	id_help
      };

    typedef std::list<std::string> list_args_type;

    /**\name Constructor and destructor 
	\{ */
  public:
    /// Construct a Cmd.
    Cmd(const identifier_type id);
    /// Destroy a Cmd.
    ~Cmd();
    /** \} */

    /** \name Accessors
	\{ */
  public:
    /// Add an argument
    void push_arg(const std::string &arg);

    /// Return the list of arguments
    const list_args_type &get_args() const;
    /// Return the Cmd type.
    identifier_type get_identifier() const;
    /** \} */

  private:
    /// The command identifier.
    identifier_type id_;

    /// List of arguments.
    list_args_type list_args_;
  };

} // namespace shell

# include "shell/cmd.hxx"

#endif // !SHELL_CMD_HH
