//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2003, 2004, 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//
/** \file vm/memory.hh
    \brief Declare Memory Management Unit class. */
#ifndef VM_MMU_HH
# define VM_MMU_HH

# include <vector>

# include "inst/fwd.hh"

# include "vm/memory.hh"
# include "vm/fwd.hh"

namespace vm
{

  /// Memory Management Unit abstraction
  class Mmu
  {
    /** \name Constructor and destructor
	\{ */
  public:
    Mmu(Cp0 &cp0, Memory &memory);
    /** \} */

    /// Reset the mmu
    void reset();

    /** \name Proxy for memory data access.
	\{ */
  protected:
    bool translate(int &offset, Memory::segment_kind &k) const;

  public:
    void data_store(const inst::DataSection& data_section);

    /// Store a byte in memory.
    void data_store_byte(int offset, int b);
    /// Store a word in memory.
    void data_store_word(int offset, int w);

  public:
    /// Load a byte from memory.
    int data_load_byte(int offset) const;
    /// Load a word from memory.
    int data_load_word(int offset) const;

    int data_sbrk(int size);
    /** \} */

    /** \name Proxy for memory instruction access.
	\{ */
  public:
    void inst_store(const inst::TextSection& text_section);

    const inst::Inst &inst_load(int offset) const;
    /** \} */

  protected:
    /// Control coprocessor link
    Cp0 &cp0_;

    /// Memory link.
    Memory &memory_;
    std::vector<const inst::Inst*> text_section_;
  };

} // namespace vm

# include "vm/mmu.hxx"

#endif // !VM_MMU_HH
